/*
Copyright (c) 2011, Sylvain Raybaud <sylvain.raybaud@gmail.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of the PAROLE team, LORIA, INRIA nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef _ctm2plain_h_
#define _ctm2plain_h_

#include <stdio.h>

#include <ngram_model.h>
#include <pocketsphinx.h>

typedef struct parameters_s {
    
    char * lmfile;
    char * dict;
    char * fdict;
    char * hmm;
    
    char ** filenames;
    
    int max_length;
    
    int seg_emit;
    int seg_prefer;
    int seg_best;
    int seg_best_prefer;
    int seg_silence;
    int seg_length;
    int seg_mix;
    int seg_none;
    
    int clean;
    int keep_fillers;
    
    ps_decoder_t * ps;
    cmd_ln_t * ps_cmd_ln;
    ngram_model_t * lm;
    int32 lm_zero;
    int32 sentence_start_id;
    int32 unk_id;
    
    /* cut on ',', '.', ';', ':', '?', '!', '</s>' */
    int32 * mt_cut_symbols_id;
    
    int verbose;
    
} parameters_t;


typedef struct word_list_s {
    char * word;
    struct word_list_s * next;
} word_list_t;

parameters_t parse_cl(int argc, char ** argv);
void parameters_free(parameters_t * p);
void init(parameters_t * params);

void ctm2plain(parameters_t * params, char * filename);

word_list_t * segment(parameters_t * params, word_list_t * stream, FILE * out);
int segment_silence(parameters_t * params, word_list_t * list);
int segment_length(parameters_t * params, word_list_t * list);
int segment_emit(parameters_t * params, word_list_t * list);
int segment_prefer(parameters_t * params, word_list_t * list);
int segment_best_prefer(parameters_t * params, word_list_t * list);
int segment_at_best(parameters_t * params, word_list_t * list);
int segment_mix(parameters_t * params, word_list_t * list);

word_list_t * write_segment(parameters_t * params, word_list_t * from, int chunk_length, FILE * out);

word_list_t * add_word(word_list_t * list, gchar * word);
word_list_t * new_word(gchar * word);
word_list_t * list_end(word_list_t * list);
word_list_t * word_list_get_first_word(word_list_t * stream);
int word_list_length(word_list_t * stream);

int is_non_word(char * word);
int is_silence(char * word);
char * clean(char * word);

#endif /* _ctm2plain_h_ */
