/*
*
* Copyright (C) 2009, Sylvain Raybaud <sylvain.raybaud@crans.org>
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of Sylvain Raybaud nor the names of its contributors may
*       be used to endorse or promote products derived from this software
*       without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Sylvain Raybaud <sylvain.raybaud@crans.org> ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL Sylvain Raybaud BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/


#include "cutils.h"

#include <string.h>
#include <stdlib.h>


/* if filename ends with ".gz" then open it through zlib o/w through stdio */
myFile myfopen(const char * filename, const char * mode) {

    myFile file = {0};

    size_t len = strlen(filename);

    if(!strcmp(&(filename[len-3]),".gz")) {
        file.gzfile = gzopen(filename,mode);
        file.gziped = 1;
        if(file.gzfile==NULL)
            file.error = 1;
        else {
            file.open = 1;
        }
    }
    else {
        file.file = fopen(filename,mode);
        if(file.file==NULL)
            file.error = 1;
        else {
            file.open = 1;
        }
    }

    if(file.open) {
        file.filename = malloc((len+1)*sizeof(char));
        strcpy(file.filename,filename);
    }

    return file;

}


int mygzscanf(gzFile file, const char * format, ...) {

    char line[MAXGZLINELENGTH];
    if(gzgets(file,line,MAXGZLINELENGTH)==Z_NULL)
        return EOF;
    else {
        va_list valist;
        va_start(valist, format);

        return vsscanf(line,format,valist);
    }
}

ssize_t mygetline(char ** lineptr, size_t * length, myFile file) {

    if(file.gziped) {
            *length = MAXGZLINELENGTH;
        *lineptr = (char*) realloc(*lineptr,MAXGZLINELENGTH);
        char * line = gzgets(file.gzfile,*lineptr,MAXGZLINELENGTH);
        return (line==Z_NULL)?-1:(ssize_t)strlen(line);
    }
    else
        return getline(lineptr,length,file.file);

}


ssize_t myclevergetline(char ** lineptr, size_t * length, myFile file) {

    int do_it = 1;
    ssize_t n = 0;

    while(do_it) {
        n = mygetline(lineptr,length,file);
        if(n==-1 || *lineptr==NULL || (*lineptr[0]!='\0' && *lineptr[0]!='\n' && *lineptr[0]!='#' && *lineptr[0]!=';'))
            do_it = 0;
    }

    return n;
}

size_t myfread(myFile f,void * ptr,size_t size, size_t n) {
    if(f.gziped) {
        int n_read = gzread(f.gzfile,ptr,(unsigned int)(size*n));
        return (n_read>=0?(size_t)n_read:0)/size;
    }
    else {
        return fread(ptr,size,n,f.file);
    }
}

int myfflush(myFile f) {
    if(!f.gziped) fflush(f.file);
}
