#include "sentence.h"

#include <stdlib.h>
#include <string.h>

#include "language_model.h"

sentence_t* sentence_new(const char *str) {
  const char *p;
  int i, count, length = strlen(str);
  sentence_t *sen;

   /* Counting words */
  count = (*str==' ' || *str=='\0' ? 0 : 1);
  for (p=str ; *p ; p++)
    if (*p==' ' && *(p+1)!=' ' && *(p+1)!='\0')
      count++;
  /* Allocating memory */
  sen = malloc(sizeof(sentence_t));
  sen->str = malloc((length+1)*sizeof(char));
  sen->wordCount = count;
  sen->words_buffer = malloc((length+1)*sizeof(char));
  sen->words = malloc((sen->wordCount*sizeof(char*)));
  sen->wids = NULL;
  sen->wids_back = NULL;

  /* Filling the structure */
  sen->wordCount = count;
  memcpy(sen->str, str, (length+1)*sizeof(char));
  memcpy(sen->words_buffer, str, (length+1)*sizeof(char));
  
  /* Tokenizing : replacing spaces in words_buffer with null characters,
     and storing pointers to words */
  if (sen->wordCount > 0)
    (sen->words)[0] = strtok(sen->words_buffer, " ");
  for (i=1 ; i<sen->wordCount ; i++)
    (sen->words)[i] = strtok(NULL, " ");
  
  return sen;
}

void sentence_free(sentence_t *sen) {
    if(sen==NULL)
        return;
    free(sen->str);
    free(sen->words_buffer);
    free(sen->words);
    free(sen->wids);
    free(sen->wids_back);
    free(sen);
}

char * sentence_get_string(sentence_t * s) {
    return s->str;
}

void sentence_set_wids(sentence_t *sen, const lmi_t *lm, const lmi_t *back_lm) {
  int i;
  wid_t wid;
  sen->wids = malloc(sen->wordCount*sizeof(wid_t));
  sen->wids_back = malloc(sen->wordCount*sizeof(wid_t));
  for (i=0 ; i<sen->wordCount ; i++) {
    wid = lm->wid(lm->data, sen->words[i]);
    sen->wids[i] = wid;
    wid = back_lm->wid(back_lm->data, sen->words[i]);
    sen->wids_back[i] = wid;
  }
}

