/*
Copyright (c) 2011, Sylvain Raybaud <sylvain.raybaud@gmail.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of the PAROLE team, LORIA, INRIA nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "maparole.h"
#include "common.h"


#define DEFAULT_HMM MODELDIR "/hmm/en_US/hub4wsj_sc_8k"
#define DEFAULT_LM MODELDIR "/lm/en/turtle.DMP"
#define DEFAULT_DICT MODELDIR "/lm/en/turtle.dic"

/* Creates a ps_cml_ln according to the configuration given in parameters,
   and stores it in deco_cfg->ps_cmd_ln */
void make_decoder_cmd_ln(mp_config_t * mp_cfg,
			       const char * cfgfilename,
			       decoder_config_t * deco_cfg) {
    cmd_ln_t * cmd_ln = NULL;
    
    /*
    CL switches override specified config files which override default config file 
    which override buil-in defaults
    */
    
    //cmd_ln = cmd_ln_init(cmd_ln,ps_args(),TRUE,"-lw", mp_cfg->ps_config.lw,NULL);
    //cmd_ln = cmd_ln_init(cmd_ln,ps_args(),TRUE,NULL);

    if(access("ps.cfg", F_OK)==0)
        cmd_ln = cmd_ln_parse_file_r(NULL,ps_args(),"ps.cfg",FALSE);
    if(cfgfilename && access(cfgfilename, F_OK)==0)
        cmd_ln = cmd_ln_parse_file_r(cmd_ln,ps_args(),cfgfilename,FALSE);
    
    if(deco_cfg->hmm_filename != NULL)
        cmd_ln = cmd_ln_init(cmd_ln,ps_args(),FALSE,"-hmm", deco_cfg->hmm_filename,NULL);
    if(deco_cfg->dict_filename != NULL)
        cmd_ln = cmd_ln_init(cmd_ln,ps_args(),FALSE,"-dict", deco_cfg->dict_filename,NULL);
    if(deco_cfg->fdict_filename != NULL)
        cmd_ln = cmd_ln_init(cmd_ln,ps_args(),FALSE,"-fdict", deco_cfg->fdict_filename,NULL);
    if(deco_cfg->mdef_filename != NULL)
        cmd_ln = cmd_ln_init(cmd_ln,ps_args(),FALSE,"-mdef", deco_cfg->mdef_filename,NULL);
    
    cmd_ln = cmd_ln_init(cmd_ln,ps_args(),FALSE,"-lw", mp_cfg->ps_config.lw_str,NULL);
    
    if (cmd_ln == NULL)
        exit(EXIT_PS_ERROR);

    deco_cfg->ps_cmd_ln = cmd_ln;
}

void mp_parse_cfg_ps(mp_config_t * cfg, const char * cfgfilename) {
  int i;
  /* If no decoder config is specified, make an empty one */
  if (cfg->ps_config.decoders.count == 0) {
    decoder_config_t c0 = {0};
    cfg->ps_config.decoders.count = 1;
    cfg->ps_config.decoders.cfg = malloc(sizeof(decoder_config_t));
    cfg->ps_config.decoders.cfg[0] = c0;
    make_decoder_cmd_ln(cfg, NULL, &cfg->ps_config.decoders.cfg[0]);
  }
  else
    for (i=0 ; i<cfg->ps_config.decoders.count ; i++)
      make_decoder_cmd_ln(cfg, NULL, &cfg->ps_config.decoders.cfg[i]);
}

int mp_check_ps_config(mp_config_t * cfg) {
    /* We'll see later */
    return 0;
}

