#ifndef _SENTENCE_H_
#define _SENTENCE_H_

union wids_s;
struct lmi_s;

/** \file
 * This file contains the sentence_t structure and related functions.
 */

/**
 * Structure providing an easy access to individual words in a sentence.
 * A word is a sequence of non-space characters.
 */
typedef struct sentence_s {
    char * str;         /**< Stores the raw sentence. */
    char * words_buffer; /**< Internal buffer : words separated by the null character. */
    /** Array indexing the words above; use this.
     s.words[i] is the i-th word of the sentence */
    char ** words;
    int wordCount;	/**< The number of words in the sentence. */

    union wid_s * wids;   /**< Array of word indices for a language model . */
    union wid_s * wids_back;  /**< Array of word indices for a backward language model . */
} sentence_t;


/** Creates a new sentence_t from string s.
 * Sets all fields but the wids ones. (Does not store s.) 
 @return a pointer to a new sentence_t */
sentence_t* sentence_new(const char *s);

/** Frees all memory allocated by sentence_new() (and sentence_set_wids() if need be). */
void sentence_free(sentence_t *s);

/** Sets the word indices for given LMs.
    Allocates memory for arrays s.wids and s.wids_back, and fill them
    using lm and back_lm respectively. */
void sentence_set_wids(sentence_t *s, const struct lmi_s *lm, const struct lmi_s *back_lm);

/** Getter */
char * sentence_get_string(sentence_t *);

#endif
