#include "fct.h"

#include "sentence.h"


FCTMF_SUITE_BGN(test_sentence_suite) {

  FCT_TEST_BGN(test_sentence_empty) {
    sentence_t *s = sentence_new("");
    fct_chk_eq_int(s->wordCount, 0);
    sentence_free(s);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_sentence_one_word) {
    char *str = "foo";
    sentence_t *s = sentence_new(str);
    fct_chk_eq_int(s->wordCount, 1);
    fct_chk_eq_str(s->words[0], "foo");
    fct_chk_eq_str(s->str, str);
    sentence_free(s);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_sentence_several_words) {
    char *str = "foo bar baz";
    sentence_t *s = sentence_new(str);
    fct_chk_eq_int(s->wordCount, 3);
    fct_chk_eq_str(s->words[0], "foo");
    fct_chk_eq_str(s->words[1], "bar");
    fct_chk_eq_str(s->words[2], "baz");
    fct_chk_eq_str(s->str, str);
    sentence_free(s);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_sentence_several_spaces) {
    char *str = "foo  bar    baz";
    sentence_t *s = sentence_new(str);
    fct_chk_eq_int(s->wordCount, 3);
    fct_chk_eq_str(s->words[0], "foo");
    fct_chk_eq_str(s->words[1], "bar");
    fct_chk_eq_str(s->words[2], "baz");
    fct_chk_eq_str(s->str, str);
    sentence_free(s);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_sentence_trailing_space) {
    char *str = "foo bar baz ";
    sentence_t *s = sentence_new(str);
    fct_chk_eq_int(s->wordCount, 3);
    fct_chk_eq_str(s->words[0], "foo");
    fct_chk_eq_str(s->words[1], "bar");
    fct_chk_eq_str(s->words[2], "baz");
    fct_chk_eq_str(s->str, str);
    sentence_free(s);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_sentence_trailing_spaces) {
    char *str = "foo bar baz  ";
    sentence_t *s = sentence_new(str);
    fct_chk_eq_int(s->wordCount, 3);
    fct_chk_eq_str(s->words[0], "foo");
    fct_chk_eq_str(s->words[1], "bar");
    fct_chk_eq_str(s->words[2], "baz");
    fct_chk_eq_str(s->str, str);
    sentence_free(s);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_sentence_opening_space) {
    char *str = " foo bar baz";
    sentence_t *s = sentence_new(str);
    fct_chk_eq_int(s->wordCount, 3);
    fct_chk_eq_str(s->words[0], "foo");
    fct_chk_eq_str(s->words[1], "bar");
    fct_chk_eq_str(s->words[2], "baz");
    fct_chk_eq_str(s->str, str);
    sentence_free(s);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_sentence_opening_spaces) {
    char *str = "   foo bar baz";
    sentence_t *s = sentence_new(str);
    fct_chk_eq_int(s->wordCount, 3);
    fct_chk_eq_str(s->words[0], "foo");
    fct_chk_eq_str(s->words[1], "bar");
    fct_chk_eq_str(s->words[2], "baz");
    fct_chk_eq_str(s->str, str);
    sentence_free(s);
  } FCT_TEST_END();

} FCTMF_SUITE_END();


