/*
Copyright (c) 2011, Sylvain Raybaud <sylvain.raybaud@gmail.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of the PAROLE team, LORIA, INRIA nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef _slf_h_
#define _slf_h_

#ifdef __cplusplus
extern "C" {
#endif

#include <pocketsphinx.h>
#include <cutils.h>


/* SLF */
/* as produced by SphinxIII */

typedef struct {
    int32 id;
    int32 from;
    int32 to;
    const char * word;
    double acoustic;
    double lm;
    int v;
    
} slf_link_t;

typedef struct slf_links_list_s {
    slf_link_t * link;
    struct slf_links_list_s * tail;
} slf_links_list_t;

typedef struct {
    int32 id;
    float t;
    char * word;
    
    slf_links_list_t * entries;
    slf_links_list_t * exits;
    
} slf_node_t;

typedef struct slf_nodes_list_s {
    slf_node_t * node;
    struct slf_nodes_list_s * tail;
} slf_nodes_list_t;

typedef struct {
    int32 nodes_count;
    int32 links_count;
    double lmscale;
    double wdpenalty;
    double base; /* default is e */
    char * utterance;
    char * lmname;
    char * version;
    
    slf_nodes_list_t * nodes;
    slf_links_list_t * links;
} slf_t;


/* PLF */
/* mimics Moses' PCN structure */
/*
typedef struct plf_scores_list_s {
    double score;
    struct plf_scores_list_s * tail;
} plf_scores_list_t;

typedef struct {
    const char * word;
    plf_scores_list_t * scores;
    int32 pos;
} plf_alt_t;

typedef struct plf_alts_list_s {
    plf_alt_t * alt;
    struct plf_alts_list_s * tail;
} plf_alts_list_t;

typedef plf_alt_list_t plf_col_t;

typedef struct plf_cols_list_s {
    plf_col_t * col;
    struct plf_cols_list_s * tail;
} plf_cols_list_t;

typedef struct {
    plf_col_t * col;
    plf_cols_list_t * tail;
} plf_t;
*/

/* methods */

slf_t * slf_load(const char * filename);
slf_t * slf_new();
void slf_free(slf_t *);

void slf_add_node(slf_t * lattice, int32 id, double time, char * word);
void slf_add_link(slf_t * lattice, int32 id, int32 from, int32 to, char * word, double a, double l, int v);



slf_link_t * slf_link_new(int32 id, int32 from, int32 to, char * word, double a, double l, int v);
slf_links_list_t * slf_links_list_get_last();
slf_links_list_t * slf_links_list_new_link();
void slf_links_list_add_link(slf_links_list_t **,slf_link_t *);
void slf_nodes_list_destroy_list_only(slf_nodes_list_t * list);


slf_node_t * slf_node_new(int32 id, double t, char * word);
slf_nodes_list_t * slf_nodes_list_get_last(slf_nodes_list_t * list);
slf_nodes_list_t * slf_nodes_list_new_node();
void slf_nodes_list_add_node(slf_nodes_list_t ** list, slf_node_t * node);
slf_node_t * slf_nodes_list_pop(slf_nodes_list_t ** queue);
slf_node_t * slf_get_node_by_id(slf_nodes_list_t * nodes_list,int32 id);
void slf_nodes_list_destroy_list_only(slf_nodes_list_t * list);


/* populate 'entries' and 'exits' lists of nodes*/
void slf_build_connections(slf_t *);

/* returns an array of topologically sorted nodes */
slf_node_t ** slf_topological_sort(slf_t *);

/*
plf_t * slf_to_plf(slf_t * lattice, const char * filename);
plf_t * ps_lattice_to_plf(ps_lattice_t * lattice, const char * filename);

int write_plf(plf_t *, const char * filename);*/


#ifdef __cplusplus
}
#endif


#endif /* _slf_h_ */
    