/*
Copyright (c) 2011, Sylvain Raybaud <sylvain.raybaud@gmail.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of the PAROLE team, LORIA, INRIA nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <iostream>
using namespace std;

#include <glib.h>
#include <pthread.h>
#include <sys/time.h>
#include <time.h>

#include <ngram_model.h>
#include <pocketsphinx.h>
#include <libxml/tree.h>

#include <cutils.h>

#include "parse_cfg.h"
#include "parse_cfg_ps.h"
#include "parse_cfg_moses.h"
#include "parse_cfg_s2tt.h"
#include "parse_cl.h"
#include "engine.h"
#include "common.h"
#include "maparole.h"
#include "interface_ps.h"
#include "interface_moses.h"
#include "confidence_measure.h"
#include "language_model.h"
#include "lattice.h"


static void print_config(mp_config_t * config) {
    fprintf(stderr,"settings:\n");
    fprintf(stderr,"  moses config: %s\n",config->cfg_moses_filename);
    fprintf(stderr,"  ps config: %s\n",config->cfg_ps_filename);
    fprintf(stderr,"  s2tt config: %s\n",config->cfg_s2tt_filename);
    fprintf(stderr,"  source lm(s):\n");
    int i;
    for(i=0;i<config->ps_config.n_lm;i++) fprintf(stderr,"    %s (%f)\n",config->ps_config.lmset[i],config->ps_config.lmweights[i]);
    fprintf(stderr,"  target lm: %s\n",config->moses_config.lm_filename);
    fprintf(stderr,"  hmm: %s\n",config->ps_config.hmm_filename);
    fprintf(stderr,"  dict: %s\n",config->ps_config.dict_filename);
    fprintf(stderr,"  asr segment: %is\n",config->asr_segment_length);
    fprintf(stderr,"  asr overlap: %is\n",config->asr_segment_overlap);
    fprintf(stderr,"  asr buffer: %i segments\n",config->asr_buffer_size);
    fprintf(stderr,"  verbose? %s\n",config->verbose?"yes":"no");
    fprintf(stderr,"\n");
}

static mp_config_t test_parse_cfg (int argc, char ** argv) {
    mp_config_t config = {0};
    
    config.asr_segment_length = 10;
    config.asr_segment_overlap = 3;
    config.asr_buffer_size = 10;
    
    config.seg_method = SEG_MIX;
    config.max_mt_chunk = 40;
    
    config.ps_config.lmbase = 1.0001;
    config.ps_config.lw = 6.5;
    config.ps_config.lw_str = "6.5";
    
    parse_cl(&config,argc,argv);
    mp_parse_cfg_s2tt(&config, config.cfg_s2tt_filename);
    mp_parse_cfg_ps(&config, config.cfg_ps_filename);
//     mp_parse_cfg_moses(&config, config.cfg_moses_filename);
    
    return config;
}

int main(int argc, char ** argv) {
    mp_config_t config = test_parse_cfg(argc,argv);
    
    print_config(&config);

    ps_decoder_t * ps = ps_init(config.ps_config.ps_cmd_ln);
    
    cerr << "LATTICE:" << endl;
    cerr <<  "  reading lattice from resources/toy.slf:" << endl;
    
    ps_lattice_t * lat = ps_lattice_read(ps,"resources/toy.slf");
    
    cerr << "  topological sort:" << endl;
    
    GSList * topo = ps_lattice_topological_sort(lat);
    for(int i = 0; i<5; i++) {
        cout << ps_latnode_baseword(lat,(ps_latnode_t*)topo->data) << endl;
        topo = g_slist_next(topo);
    }
    
    
//     mp_start_translation(&config,&engine);
    
    return 0;
    
}
