#include "fct.h"

#include <math.h>

#include "language_model.h"
#include "interface_moses.h"

/* error on log score */
static const float EPSILON = 0.05;


#define LENGTH(tab) ((int)(sizeof(tab)/sizeof(tab[0])))

/* Those arrays hold the content of tests/resources/toy.arpa */
static struct unigram_s {
  float score;
  char *word;
} unigrams_tab[] = 
  {
    {-1.2430, "</s>"},
    {-1.2430, "<s>"},
    {-0.8451, "a"},
    {-1.2430, "are"},
    {-0.8451, "few"},
    {-1.2430, "is"},
    {-1.2430, "made"},
    {-1.2430, "of"},
    {-1.2430, "repeated"},
    {-1.2430, "sentence"},
    {-1.2430, "this"},
    {-1.2430, "where"},
    {-0.8451, "words"}
  };
static struct bigram_s {
  float score;
  char *w1;
  char *w2;
} bigrams_tab[] = 
  {
    {-0.1761, "<s>", "this"},
    {-0.3979, "a", "few"},
    {-0.3979, "a", "sentence"},
    {-0.1761, "are", "repeated"},
    {-0.1249, "few", "words"},
    {-0.1761, "is", "a"},
    {-0.1761, "made", "of"},
    {-0.1761, "of", "few"},
    {-0.1761, "sentence", "made"},
    {-0.1761, "this", "is"},
    {-0.1761, "where", "a"},
    {-0.3979, "words", "are"},
    {-0.3979, "words", "where"}
  };
static struct trigram_s {
  float score;
  char *w1;
  char *w2;
  char *w3;
} trigrams_tab[] = 
  {
    {-0.3010, "<s>", "this", "is"},
    {-0.3010, "a", "few", "words"},
    {-0.3010, "a", "sentence", "made"},
    {-0.3010, "are", "repeated", "</s>"},
    {-0.4771, "few", "words", "are"},
    {-0.4771, "few", "words", "where"},
    {-0.3010, "is", "a", "sentence"},
    {-0.3010, "made", "of", "few"},
    {-0.3010, "of", "few", "words"},
    {-0.3010, "sentence", "made", "of"},
    {-0.3010, "this", "is", "a"},
    {-0.3010, "where", "a", "few"},
    {-0.3010, "words", "are", "repeated"},
    {-0.3010, "words", "where", "a"}
  };


/* Test data for ngrams non existing in toy.arpa */
static char * absent_unigram[] = {"foo", "thi", "madeo"};
static char * absent_bigram[] = {
  "fewmlk", "words",
  "few", "wordsmoij",
  "fewmlk", "wordosids"
};
static char * absent_trigram[] = {
  "amoij", "few", "words",
  "a", "fewmoi", "words",
  "a", "few", "miljlwords",
  "mmimoa", "ooimjfew", "wordsomi",
  "a", "words", "few"
};


/* Questionnable hack to achieve a "static" setup/teardown */
static int first = 1, final = 0;

FCTMF_FIXTURE_SUITE_BGN(test_lm_suite) {

  /* testing functions parametered by the language model */

  void test_unigram_score(lmi_t *lm) {
    int i;
    for (i=0 ; i<LENGTH(unigrams_tab) ; i++) {
      float a = log(lm->score_unigram(lm->data, lm->wid(lm->data,unigrams_tab[i].word)))/M_LN10;
      float b = unigrams_tab[i].score;
      float delta = fabs(a - b);
      fct_xchk(((int)(delta < EPSILON)), "failed unigram score (\"%s\"), delta = %f",
	       unigrams_tab[i].word, delta);
    }
  }

  void test_bigram_score(lmi_t *lm) {
    int i;
    for (i=0 ; i<LENGTH(bigrams_tab) ; i++) {
      float a = log(lm->score_bigram(lm->data,
				     lm->wid(lm->data,bigrams_tab[i].w1),
				     lm->wid(lm->data,bigrams_tab[i].w2))
		    )/M_LN10;
      float b = bigrams_tab[i].score;
      float delta = fabs(a - b);
      fct_xchk(((int)(delta < EPSILON)), "failed bigram score (\"%s %s\"), delta = %f",
	       bigrams_tab[i].w1, bigrams_tab[i].w2, delta);
    }
  }

  void test_trigram_score(lmi_t *lm) {
    int i;
    for (i=0 ; i<LENGTH(trigrams_tab) ; i++) {
      float a = log(lm->score_trigram(lm->data,
				      lm->wid(lm->data,trigrams_tab[i].w1),
				      lm->wid(lm->data,trigrams_tab[i].w2),
				      lm->wid(lm->data,trigrams_tab[i].w3))
		    )/M_LN10;
      float b = trigrams_tab[i].score;
      float delta = fabs(a - b);
      fct_xchk(((int)(delta < EPSILON)), "failed trigram score (\"%s %s %s\"), delta = %f",
	       trigrams_tab[i].w1, trigrams_tab[i].w2, trigrams_tab[i].w3, delta);
    }
  }

  void test_existing_unigram(lmi_t *lm) {
    int i;
    for (i=0 ; i<LENGTH(unigrams_tab) ; i++) {
      int exists = lm->unigram_exists(lm->data, lm->wid(lm->data,unigrams_tab[i].word));
      fct_xchk(exists, "failed on existing unigram (\"%s\")",
	       unigrams_tab[i].word);
    }
  }

  void test_absent_unigram(lmi_t *lm) {
    int i;
    for (i=0 ; i<LENGTH(absent_unigram) ; i++) {
      int exists = lm->unigram_exists(lm->data, lm->wid(lm->data,absent_unigram[i]));
      fct_xchk(!exists, "failed on absent unigram (\"%s\")",
	       absent_unigram[i]);
    }
  }
  
  void test_existing_bigram(lmi_t *lm) {
    int i;
    for (i=0 ; i<LENGTH(bigrams_tab) ; i++) {
      int exists = lm->bigram_exists(lm->data,
				     lm->wid(lm->data,bigrams_tab[i].w1),
				     lm->wid(lm->data,bigrams_tab[i].w2));
      fct_xchk(exists, "failed on existing bigram (\"%s %s\")",
	       bigrams_tab[i].w1, bigrams_tab[i].w2);
    }
  }

  void test_absent_bigram(lmi_t *lm) {
    int i;
    for (i=0 ; i<LENGTH(absent_bigram) ; i+=2) {
      int exists = lm->bigram_exists(lm->data,
				     lm->wid(lm->data,absent_bigram[i]),
				     lm->wid(lm->data,absent_bigram[i+1]));
      fct_xchk(!exists, "failed on absent bigram (\"%s %s\")",
	       absent_bigram[i], absent_bigram[i+1]);
    }
  }
  
  void test_existing_trigram(lmi_t *lm) {
    int i;
    for (i=0 ; i<LENGTH(trigrams_tab) ; i++) {
      int exists = lm->trigram_exists(lm->data,
				      lm->wid(lm->data,trigrams_tab[i].w1),
				      lm->wid(lm->data,trigrams_tab[i].w2),
				      lm->wid(lm->data,trigrams_tab[i].w3));
      fct_xchk(exists, "failed on existing trigram (\"%s %s %s\")",
	       trigrams_tab[i].w1, trigrams_tab[i].w2, trigrams_tab[i].w3);
    }
  }

  void test_absent_trigram(lmi_t *lm) {
    int i;
    for (i=0 ; i<LENGTH(absent_trigram) ; i+=3) {
      int exists = lm->trigram_exists(lm->data,
				      lm->wid(lm->data,absent_trigram[i]),
				      lm->wid(lm->data,absent_trigram[i+1]),
				      lm->wid(lm->data,absent_trigram[i+2]));
      fct_xchk(!exists, "failed on absent trigram (\"%s %s %s\")",
	       absent_trigram[i], absent_trigram[i+1], absent_trigram[i+2]);
    }
  }


  logmath_t *lmath = NULL;
  sphinx_lm_t *sphinx_data = NULL;
  lmi_t sphinx_lm;
    
  moses_config_t cfg;
  moses_lm_t *moses_data = NULL;
  lmi_t moses_lm;

  FCT_SETUP_BGN() {
    if (first) {
      lmath = logmath_init(1.1, 0, 0);
      sphinx_data = sphinx_open_lm("tests/resources/toy.arpa", lmath);
      sphinx_lm = sphinx_lmi_new(sphinx_data);
      
      moses_read_config_file("tests/resources/moses.test_lm.ini", &cfg);
      moses_static_data(&cfg);
      moses_data = moses_get_lm();
      moses_lm = moses_lmi_new(moses_data);
      first = 0;
    }
  } FCT_SETUP_END();
  
  FCT_TEARDOWN_BGN() {
    if (final) {
      sphinx_lm.free_data(sphinx_lm.data);
      free(sphinx_lm.data);
      moses_terminate(&cfg);
      logmath_free(lmath);
    }
  } FCT_TEARDOWN_END();



  /* Test suite */

  FCT_TEST_BGN(test_lm_unigram_score_sphinx) {
    test_unigram_score(&sphinx_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_bigram_score_sphinx) {
    test_bigram_score(&sphinx_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_trigram_score_sphinx) {
    test_trigram_score(&sphinx_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_existing_unigram_sphinx) {
    test_existing_unigram(&sphinx_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_absent_unigram_sphinx) {
    test_absent_unigram(&sphinx_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_existing_bigram_sphinx) {
    test_existing_bigram(&sphinx_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_absent_bigram_sphinx) {
    test_absent_bigram(&sphinx_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_existing_trigram_sphinx) {
    test_existing_trigram(&sphinx_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_absent_trigram_sphinx) {
    test_absent_trigram(&sphinx_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_unigram_score_moses) {
    test_unigram_score(&moses_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_bigram_score_moses) {
    test_bigram_score(&moses_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_trigram_score_moses) {
    test_trigram_score(&moses_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_existing_unigram_moses) {
    test_existing_unigram(&moses_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_absent_unigram_moses) {
    test_absent_unigram(&moses_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_existing_bigram_moses) {
    test_existing_bigram(&moses_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_absent_bigram_moses) {
    test_absent_bigram(&moses_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_existing_trigram_moses) {
    test_existing_trigram(&moses_lm);
  } FCT_TEST_END();

  FCT_TEST_BGN(test_lm_absent_trigram_moses) {
    test_absent_trigram(&moses_lm);
    final = 1;
  } FCT_TEST_END();

} FCTMF_FIXTURE_SUITE_END();



