/*
Copyright (c) 2011, Sylvain Raybaud <sylvain.raybaud@gmail.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of the PAROLE team, LORIA, INRIA nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <pocketsphinx.h>

#include <cutils.h>

#include "common.h"
#include "maparole.h"
#include "interface_ps.h"

const char * ps_hyp_from_raw(mp_config_t * config, mp_engine_t * engine) {
    
    FILE * audiofile;
    audiofile = fopen(config->audiofilename, "rb");
    
    if (audiofile == NULL) {
        perror("Failed to open audio file");
        exit(EXIT_PS_ERROR);
    }
    
    int ps_return = ps_decode_raw(engine->ps, audiofile, config->audiofilename, -1);
    myverbose(config->verbose,"PS: read %i samples from '%s'",ps_return,config->audiofilename);
    if (ps_return < 0) {
        myfail(EXIT_PS_ERROR,"Could not decode audiofile '%s'",config->audiofilename);
    }
    
    char const * uttid;
    int32 score;
    
    const char * hyp;
    
    hyp = ps_get_hyp(engine->ps, &score, &uttid);
    if (hyp == NULL) {
        myfail(EXIT_PS_ERROR,"Could not get utterance from '%s'",config->audiofilename);
    }
    
    return hyp;
}
