/*
Copyright (c) 2011, Sylvain Raybaud <sylvain.raybaud@gmail.com>
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of the PAROLE team, LORIA, INRIA nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include <glib.h>
#include <stdio.h>

#include <cutils.h>

#include "parse_cfg.h"
#include "parse_cfg_ps.h"
#include "parse_cfg_moses.h"
#include "parse_cfg_s2tt.h"
#include "parse_cl.h"
#include "common.h"
#include "maparole.h"


mp_config_t parse_cfg(int argc, char ** argv) {
    
    mp_config_t config = {0};
    
    config.asr_segment_length = 10;
    config.asr_segment_overlap = 3;
    config.asr_buffer_size = 10;
    
    config.seg_method = SEG_MIX;
    config.max_mt_chunk = 40;
    
    config.ps_config.lmbase = 1.0001;
    
    parse_cl(&config,argc,argv);
    mp_parse_cfg_s2tt(&config, config.cfg_s2tt_filename);
    mp_parse_cfg_ps(&config, config.cfg_ps_filename);
    mp_parse_cfg_moses(&config, config.cfg_moses_filename);
    
    check_config(&config);
    
    return config;
    
}

void check_config(mp_config_t * cfg) {
    
    int pscfg = mp_check_ps_config(cfg);
    int mosescfg = mp_check_moses_config(cfg);
    int s2ttcfg = mp_check_s2tt_config(cfg);
    
    if(pscfg)
        myerror("Invalid PocketSphinx configuration");
    if(mosescfg)
        myerror("Invalid Moses configuration");
    if(s2ttcfg)
        myerror("Invalid S2TT configuration");
    
    int code = pscfg || mosescfg || s2ttcfg;
    
    if(code)
        myfail(EXIT_CONFIG_ERROR,"Invalid configuration (see above message(s)).");
    
}
