#include "confidence_measure.h"

#include <string.h>
#include <stdlib.h>
#include <fann.h>

#include "common.h"
#include "engine.h"
#include "sentence.h"

/********************/
/*** Declarations ***/
/********************/

//FIXME: are these names the names in the paper?
//TODO: cite the paper (and maybe a few others) in the comments
/* MT word level features */
float mt_x21(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float mt_x21_left(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float mt_x21_center(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float mt_x21_right(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float mt_x22(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float mt_x22_left(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float mt_x22_center(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float mt_x22_right(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float mt_x23(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float mt_x23_left(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float mt_x23_center(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float mt_x23_right(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);

/* ASR word level features */
float asr_x21(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float asr_x21_left(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float asr_x21_center(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float asr_x21_right(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float asr_x22(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float asr_x22_left(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float asr_x22_center(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float asr_x22_right(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float asr_x23(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float asr_x23_left(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float asr_x23_center(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);
float asr_x23_right(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i);

/* MT sentence level features */
float mt_x30(cm_resources_t *res, feature_set_t *set, translation_pair_t *info);
float mt_x31(cm_resources_t *res, feature_set_t *set, translation_pair_t *info);
float mt_x32(cm_resources_t *res, feature_set_t *set, translation_pair_t *info);
/* ASR sentence level features */
float asr_x30(cm_resources_t *res, feature_set_t *set, translation_pair_t *info);
float asr_x31(cm_resources_t *res, feature_set_t *set, translation_pair_t *info);
float asr_x32(cm_resources_t *res, feature_set_t *set, translation_pair_t *info);


/* Utils */
static feature_set_t * feature_set_new_empty();
static void map_features(feature_set_t *set);
static void set_counts(feature_set_t *set);
static void set_totals(feature_set_t *set);
static void enable_all_features(feature_set_t *set);
static void enable_features_by_name(feature_set_t *set, char **names);


/*********************************************************/
/*** Hard-coded static lists of the available features ***/
/*********************************************************/

/* Those are meants to be copied to create runtime lists,
 * hence we don't set the 'enabled' or 'idx' fields. */
/* /!\ Each enum must match exactly the content of the related array. */

/*  MT word level features */
static word_level_feature_t mt_wlf[] = {
  {mt_x21, "mt_x21"},
  {mt_x21_left, "mt_x21_left"},
  {mt_x21_center, "mt_x21_center"},
  {mt_x21_right, "mt_x21_right"},
  {mt_x22, "mt_x22"},
  {mt_x22_left, "mt_x22_left"},
  {mt_x22_center, "mt_x22_center"},
  {mt_x22_right, "mt_x22_right"},
  {mt_x23, "mt_x23"},
  {mt_x23_left, "mt_x23_left"},
  {mt_x23_center, "mt_x23_center"},
  {mt_x23_right, "mt_x23_right"},
};
/* enum for convenient static array access; because some features use the results of other ones
 * e.g. feature mt_x21 is accessible as mt_wlf[MT_X21],
 * and the result of the feature should be stored at index mt_wlf[MT_X21].idx of an array */
enum mt_wordLevelFeature {
  MT_X21,MT_X21_LEFT,MT_X21_CENTER,MT_X21_RIGHT,
  MT_X22,MT_X22_LEFT,MT_X22_CENTER,MT_X22_RIGHT,
  MT_X23,MT_X23_LEFT,MT_X23_CENTER,MT_X23_RIGHT,
};

/* MT sentence level features*/
static sent_level_feature_t mt_slf[] = {
  {mt_x30, "mt_x30"},
  {mt_x31, "mt_x31"},
  {mt_x32, "mt_x32"},
};
enum mt_sentLevelFeatures{
  MT_X30, MT_X31, MT_X32,
};

/* ASR word level features*/
static word_level_feature_t asr_wlf[] = {
  {asr_x21, "asr_x21"},
  {asr_x21_left, "asr_x21_left"},
  {asr_x21_center, "asr_x21_center"},
  {asr_x21_right, "asr_x21_right"},
  {asr_x22, "asr_x22"},
  {asr_x22_left, "asr_x22_left"},
  {asr_x22_center, "asr_x22_center"},
  {asr_x22_right, "asr_x22_right"},
  {asr_x23, "asr_x23"},
  {asr_x23_left, "asr_x23_left"},
  {asr_x23_center, "asr_x23_center"},
  {asr_x23_right, "asr_x23_right"},
};
enum asr_wordLevelFeature {
  ASR_X21,ASR_X21_LEFT,ASR_X21_CENTER,ASR_X21_RIGHT,
  ASR_X22,ASR_X22_LEFT,ASR_X22_CENTER,ASR_X22_RIGHT,
  ASR_X23,ASR_X23_LEFT,ASR_X23_CENTER,ASR_X23_RIGHT,
};

/* ASR sentence level features*/
static sent_level_feature_t asr_slf[] = {
  {asr_x30, "asr_x30"},
  {asr_x31, "asr_x31"},
  {asr_x32, "asr_x32"},
};
enum asr_sentLevelFeatures{
  ASR_X30, ASR_X31, ASR_X32,
};


/**************************/
/*** "public" functions ***/
/**************************/

cm_resources_t *cm_resources_new(struct cm_config_s *config, sphinx_lm_t lm) {
    /* Config check */
    if (config->mt_backward_lm_filename == NULL )
        myfail(EXIT_CONFIG_ERROR, "No backward target LM file specified.");
    if (config->asr_backward_lm_filename == NULL )
        myfail(EXIT_CONFIG_ERROR, "No backward source LM specified.");
    if (config->mt_wlf_fann_filename == NULL)
        myfail(EXIT_CONFIG_ERROR, "No fann file specified for word level confidence measures");
    if (config->mt_slf_fann_filename == NULL)
        myfail(EXIT_CONFIG_ERROR, "No fann file specified for sentence level confidence measures");

    cm_resources_t *res = malloc(sizeof(cm_resources_t));

    /* Source LM from Sphinx's instance */
    sphinx_lm_t *lmi = malloc(sizeof(sphinx_lm_t));
    memcpy (lmi, &lm, sizeof(sphinx_lm_t));
    res->asr_lm = sphinx_lmi_new(lmi);
    /* Source backward LM from file */
    res->asr_backward_lm = sphinx_lmi_new(sphinx_lm_open(config->asr_backward_lm_filename,
                                lmi->lmath));
    /* Target LM from Moses' instance */
    res->mt_lm = moses_lmi_new(moses_get_lm());
    /* Target backward LM from file */
    res->mt_backward_lm = moses_lmi_new(moses_lm_open(config->mt_backward_lm_filename));
    
    res->mt_word_level_ann = fann_create_from_file(config->mt_wlf_fann_filename);
    if (res->mt_word_level_ann == NULL)
        myfail(EXIT_UNKNOWN_ERROR, "Enable to create neural network from file '%s'", config->mt_wlf_fann_filename);
    res->mt_sent_level_ann = fann_create_from_file(config->mt_slf_fann_filename);
    if (res->mt_sent_level_ann == NULL)
        myfail(EXIT_UNKNOWN_ERROR, "Enable to create neural network from file '%s'", config->mt_slf_fann_filename);
    res->asr_word_level_ann = fann_create_from_file(config->asr_wlf_fann_filename);
    if (res->asr_word_level_ann == NULL)
        myfail(EXIT_UNKNOWN_ERROR, "Enable to create neural network from file '%s'", config->asr_wlf_fann_filename);
    res->asr_sent_level_ann = fann_create_from_file(config->asr_slf_fann_filename);
    if (res->asr_sent_level_ann == NULL)
        myfail(EXIT_UNKNOWN_ERROR, "Enable to create neural network from file '%s'", config->asr_slf_fann_filename);
    return res;
}

void cm_resources_free(cm_resources_t *res) {
    debug("start");
    
    if(!res)
        return;
    
    if(res->asr_backward_lm.data) {
        res->asr_backward_lm.free_data(res->asr_backward_lm.data);
        free(res->asr_backward_lm.data);
    }
    if(res->asr_lm.data)         free(res->asr_lm.data);
    if(res->mt_backward_lm.data) res->mt_backward_lm.free_data(res->mt_backward_lm.data);
    if(res->mt_word_level_ann)   fann_destroy(res->mt_word_level_ann);
    if(res->mt_sent_level_ann)   fann_destroy(res->mt_sent_level_ann);
    if(res->asr_word_level_ann)  fann_destroy(res->asr_word_level_ann);
    if(res->asr_sent_level_ann)  fann_destroy(res->asr_sent_level_ann);
    free(res);
}

void print_features(feature_set_t *features, translation_pair_t *info) {
  int i, j;
  printf("Source is: %s\n", info->source->str);
  printf("ASR word level features\n");
  for (i=0 ; i<info->source->wordCount ; i++) {
    printf("%s\n", info->source->words[i]);
    for (j=0 ; j<features->asr_wlf_count ; j++)
      printf("%+10.3e ", info->asr_wordLevelFeatures[i][j]);
    printf("\n");
  }

  printf("ASR sentence level features\n");
  for (j=0 ; j<features->asr_slf_count ; j++)
    printf("%+10.3e ", info->asr_sentLevelFeatures[j]);
  printf("\n");

  printf("\n");

  printf("Target is: %s\n", info->target->str);
  printf("MT word level features\n");
  for (i=0 ; i<info->target->wordCount ; i++) {
    printf("%s\n", info->target->words[i]);
    for (j=0 ; j<features->mt_wlf_count ; j++)
      printf("%+10.3e ", info->mt_wordLevelFeatures[i][j]);
    printf("\n");
  }

  printf("MT sentence level features\n");
  for (j=0 ; j<features->mt_slf_count ; j++)
    printf("%+10.3e ", info->mt_sentLevelFeatures[j]);
  printf("\n");
}

feature_set_t * feature_set_new() {
  feature_set_t *set = feature_set_new_empty();
  enable_all_features(set);
  set_counts(set);
  map_features(set);
  return set;
}

feature_set_t * feature_set_new_by_name(char **names) {
  feature_set_t *set = feature_set_new_empty();
  enable_features_by_name(set, names);
  set_counts(set);
  map_features(set);
  return set;
}

void feature_set_free(feature_set_t *set) {
  free(set->mt_wlf);
  free(set->mt_slf);
  free(set->asr_wlf);
  free(set->asr_slf);
  free(set);
}

void compute_features(feature_set_t *set, cm_resources_t *res, translation_pair_t *info) {
  int i=0, j=0;
  if (info->mt_wordLevelFeaturesBuffer != NULL)
    mywarning("compute_features() may have been called already");
  else
    translation_pair_alloc_features(set, info);

  sentence_set_wids(info->source, &res->asr_lm, &res->asr_backward_lm);
  sentence_set_wids(info->target, &res->mt_lm, &res->mt_backward_lm);

  /* ASR features */
  pthread_mutex_lock(&((sphinx_lm_t*)res->asr_lm.data)->mutex);
  pthread_mutex_lock(&((sphinx_lm_t*)res->asr_backward_lm.data)->mutex);
  for (j=0 ; j<set->asr_wlf_total ; j++)
    if (set->asr_wlf[j].enabled)
      for (i=0 ; i<info->source->wordCount ; i++)
	info->asr_wordLevelFeatures[i][set->asr_wlf[j].idx] = set->asr_wlf[j].fun(res, set, info, i);
  /* sentence level features are computed after word level ones since they may depend on it (x32) */
  for (j=0 ; j<set->asr_slf_total ; j++)
    if (set->asr_slf[j].enabled)
      info->asr_sentLevelFeatures[set->asr_slf[j].idx] = set->asr_slf[j].fun(res, set, info);
  pthread_mutex_unlock(&((sphinx_lm_t*)res->asr_lm.data)->mutex);
  pthread_mutex_unlock(&((sphinx_lm_t*)res->asr_backward_lm.data)->mutex);
  /* feed them into the suitable neural network */
  info->asr_sent_cm = *fann_run(res->asr_sent_level_ann, info->asr_sentLevelFeatures);
  for (i=0 ; i<info->source->wordCount ; i++)
    info->asr_word_cm[i] = *fann_run(res->asr_word_level_ann, info->asr_wordLevelFeatures[i]);

  /* MT features */
  for (j=0 ; j<set->mt_wlf_total ; j++)
    if (set->mt_wlf[j].enabled)
      for (i=0 ; i<info->target->wordCount ; i++)
	info->mt_wordLevelFeatures[i][set->mt_wlf[j].idx] = set->mt_wlf[j].fun(res, set, info, i);
  for (j=0 ; j<set->mt_slf_total ; j++)
    if (set->mt_slf[j].enabled)
      info->mt_sentLevelFeatures[set->mt_slf[j].idx] = set->mt_slf[j].fun(res, set, info);
  /* feed them into the suitable neural network */
  info->mt_sent_cm = *fann_run(res->mt_sent_level_ann, info->mt_sentLevelFeatures);
  for (i=0 ; i<info->target->wordCount ; i++)
    info->mt_word_cm[i] = *fann_run(res->mt_word_level_ann, info->mt_wordLevelFeatures[i]);
}


/**************************************/
/*** Utils for the *_new* functions ***/
/**************************************/


static feature_set_t * feature_set_new_empty() {
  feature_set_t *set = malloc(sizeof(feature_set_t));
  set->mt_wlf = malloc(sizeof(mt_wlf));
  memcpy(set->mt_wlf, mt_wlf, sizeof(mt_wlf));
  set->mt_slf = malloc(sizeof(mt_slf));
  memcpy(set->mt_slf, mt_slf, sizeof(mt_slf));
  set->asr_wlf = malloc(sizeof(asr_wlf));
  memcpy(set->asr_wlf, asr_wlf, sizeof(asr_wlf));
  set->asr_slf = malloc(sizeof(asr_slf));
  memcpy(set->asr_slf, asr_slf, sizeof(asr_slf));
  set_totals(set);
  return set;
}
static void enable_all_features(feature_set_t *set) {
  int i;
  for (i=0 ; i< set->mt_wlf_total ; i++)
    set->mt_wlf[i].enabled = 1;
  for (i=0 ; i< set->mt_slf_total ; i++)
    set->mt_slf[i].enabled = 1;
  for (i=0 ; i< set->asr_wlf_total ; i++)
    set->asr_wlf[i].enabled = 1;
  for (i=0 ; i< set->asr_slf_total ; i++)
    set->asr_slf[i].enabled = 1;
}
static void enable_features_by_name(feature_set_t *set, char **names) {
  int i;
  char **p = names;
  while (*p != NULL) {
    for (i=0 ; i< set->mt_wlf_total ; i++)
      if (strcmp(set->mt_wlf[i].name, *p) == 0) {
    	set->mt_wlf[i].enabled = 1;
	goto next_iter;
      }
    for (i=0 ; i< set->mt_slf_total ; i++)
      if (strcmp(set->mt_slf[i].name, *p) == 0) {
    	set->mt_slf[i].enabled = 1;
	goto next_iter;
      }
    for (i=0 ; i< set->asr_wlf_total ; i++) {
      if (strcmp(set->asr_wlf[i].name, *p) == 0)
    	set->asr_wlf[i].enabled = 1;
	goto next_iter;
      }
    for (i=0 ; i< set->asr_slf_total ; i++) {
      if (strcmp(set->asr_slf[i].name, *p) == 0)
    	set->asr_slf[i].enabled = 1;
	goto next_iter;
      }
    mywarning("Feature '%s' does not exist.", *p);
  next_iter: p++;
  }
}
/* Set the *count variables of 'set' to the correct value */
static void set_counts(feature_set_t *set) {
  int i;
  for (set->mt_wlf_count=0, i=0 ; i< set->mt_wlf_total ; i++)
    if (set->mt_wlf[i].enabled)
      set->mt_wlf_count++;
  for (set->mt_slf_count=0, i=0 ; i< set->mt_slf_total ; i++)
    if (set->mt_slf[i].enabled)
      set->mt_slf_count++;
  for (set->asr_wlf_count=0, i=0 ; i< set->asr_wlf_total ; i++)
    if (set->asr_wlf[i].enabled)
      set->asr_wlf_count++;
  for (set->asr_slf_count=0, i=0 ; i< set->asr_slf_total ; i++)
    if (set->asr_slf[i].enabled)
      set->asr_slf_count++;
}
/* Set the *total variables of 'set' to the correct value */
static void set_totals(feature_set_t *set) {
  set->mt_wlf_total = sizeof(mt_wlf)/sizeof(mt_wlf[0]);
  set->mt_slf_total = sizeof(mt_slf)/sizeof(mt_slf[0]);
  set->asr_wlf_total = sizeof(asr_wlf)/sizeof(asr_wlf[0]);
  set->asr_slf_total = sizeof(asr_slf)/sizeof(asr_slf[0]);
}
/* Set the idx variables of every feature of 'set' to the correct value */
static void map_features(feature_set_t *set) {
  int i, j;
  for (i=0, j=0; i< set->mt_wlf_total ; i++)
    if (set->mt_wlf[i].enabled)
      set->mt_wlf[i].idx = j++;
    else
      set->mt_wlf[i].idx = -1;
  for (i=0, j=0; i< set->mt_slf_total ; i++)
    if (set->mt_slf[i].enabled)
      set->mt_slf[i].idx = j++;
    else
      set->mt_slf[i].idx = -1;
  for (i=0, j=0; i< set->asr_wlf_total ; i++)
    if (set->asr_wlf[i].enabled)
      set->asr_wlf[i].idx = j++;
    else
      set->asr_wlf[i].idx = -1;
  for (i=0, j=0; i< set->asr_slf_total ; i++)
    if (set->asr_slf[i].enabled)
      set->asr_slf[i].idx = j++;
    else
      set->asr_slf[i].idx = -1;
}


/*******************************************************************************/
/*** From here, all the implementations of the feature computation functions ***/
/*******************************************************************************/


/* x21 feature : a generic function, plus two wrappers for MT and ASR */
float x21(lmi_t *lm, sentence_t *sen, int i) {
  wid_t *t = sen->wids;
  if (i==0)
    return lm->score_bigram(lm->data, lm->bos, t[i]);
  else if (i==1)
    return lm->score_trigram(lm->data, lm->bos, t[i-1], t[i]);
  else
    return lm->score_trigram(lm->data, t[i-2], t[i-1], t[i]); 
}
float mt_x21(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i) {
  return x21(&res->mt_lm, info->target, i);
}
float asr_x21(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i) {
  return x21(&res->asr_lm, info->source, i);
}


float x22(lmi_t *lm, sentence_t *sen, int i) {
  wid_t *t = sen->wids;
  if (i==sen->wordCount-1)
    return lm->score_bigram(lm->data, lm->eos, t[i]);
  else if (i==sen->wordCount-2)
    return lm->score_trigram(lm->data, lm->eos, t[i+1], t[i]);
  else
    return lm->score_trigram(lm->data, t[i+2], t[i+1], t[i]);
}
float mt_x22(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i) {
  return x22(&res->mt_lm, info->target, i);
}
float asr_x22(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i) {
  return x22(&res->asr_lm, info->source, i);
}

float x23(lmi_t *lm, sentence_t *sen, int i) {
  wid_t *t = sen->wids;
  /* if first word */
  if (i==0) {
    if (lm->unigram_exists(lm->data, t[0]))
      return 1.0;
    else
      return 0.1;
  }
  /* if second word */
  else if (i==1) {
    if (lm->bigram_exists(lm->data, t[0], t[1]))
      return 1.0;
    else if (!lm->unigram_exists(lm->data, t[1]))
      return 0.1;
    else
      if (lm->unigram_exists(lm->data, t[0]))
	return 0.6;
      else
	return 0.2;
  }
  /* if at least third word */
  else {
    if (lm->trigram_exists(lm->data, t[i-2], t[i-1], t[i]))
      return 1.;
    else if (lm->bigram_exists(lm->data, t[i-2], t[i-1]) && lm->bigram_exists(lm->data, t[i-1], t[i]))
      return 0.8;
    else if (lm->bigram_exists(lm->data, t[i-1], t[i]))
      return 0.6;
    else if (lm->bigram_exists(lm->data, t[i-2], t[i-1]) && lm->unigram_exists(lm->data, t[i]))
      return 0.4;
    else if (lm->unigram_exists(lm->data, t[i-1]) && lm->unigram_exists(lm->data, t[i]))
      return 0.3;
    else if (lm->unigram_exists(lm->data, t[i]))
      return 0.2;
    else
      return 0.1;
  }
}
float mt_x23(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i) {
  return x23(&res->mt_lm, info->target, i);
}
float asr_x23(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i) {
  return x23(&res->asr_lm, info->source, i);
}

float x30(lmi_t *lm, sentence_t *s, int J) {
  int j;
  double p = 1.0;
  if (J>1)
    p *= lm->score_bigram(lm->data, lm->bos, s->wids[0]);
  if (J>2)
    p *= lm->score_trigram(lm->data, lm->bos, s->wids[0], s->wids[1]);
  for (j=2 ; j<J ; j++) {
    p *= lm->score_trigram(lm->data, s->wids[j-2], s->wids[j-1], s->wids[j]);
  }
  if (J>1) {
    p *= lm->score_trigram(lm->data, s->wids[J-2], s->wids[J-1], lm->eos);
  }
  else if (J>0) {
    p *= lm->score_bigram(lm->data, s->wids[J-1], lm->eos);
  }
  if (J != 0)
    p = pow(p, 1./J);
  else
    p = 0;
  return p;
}
float mt_x30(cm_resources_t *res, feature_set_t *set, translation_pair_t *info) {
  return x30(&res->mt_lm, info->target, info->target->wordCount);
}
float asr_x30(cm_resources_t *res, feature_set_t *set, translation_pair_t *info) {
  return x30(&res->asr_lm, info->source, info->source->wordCount);
}


float x31(lmi_t *lm, sentence_t *s, int J) {
  int j;
  double p = 1.0;
  if (J>1)
    p *= lm->score_bigram(lm->data, lm->eos, s->wids_back[J-1]);
  if (J>2)
    p *= lm->score_trigram(lm->data, lm->eos, s->wids_back[J-1], s->wids_back[J-2]);
  for (j=J-3 ; j>0 ; j--)
    p *= lm->score_trigram(lm->data, s->wids_back[j+2], s->wids_back[j+1], s->wids_back[j]);

  if (J>1) {
    p *= lm->score_trigram(lm->data, s->wids_back[1], s->wids_back[0], lm->bos);
  }
  else if (J>0)
    p *= lm->score_bigram(lm->data, s->wids_back[0], lm->bos);

  if (J != 0)
    p = pow(p, 1./J);
  else
    p = 0;
  return p;
}
float mt_x31(cm_resources_t *res, feature_set_t *set, translation_pair_t *info) {
  return x31(&res->mt_backward_lm, info->target, info->target->wordCount);
}
float asr_x31(cm_resources_t *res, feature_set_t *set, translation_pair_t *info) {
  return x31(&res->asr_backward_lm, info->source, info->source->wordCount);
}

float mt_x32(cm_resources_t *res, feature_set_t *set, translation_pair_t *info) {
  int j, x23 = set->mt_wlf[MT_X23].idx;
  int J = info->target->wordCount;
  double p = 0;
  for (j=0 ; j<J ; j++)
    p += info->mt_wordLevelFeatures[j][x23];
  if (J > 0)
    p /= J;
  return p;
}

float asr_x32(cm_resources_t *res, feature_set_t *set, translation_pair_t *info) {
  int j, x23 = set->asr_wlf[ASR_X23].idx;
  int J = info->source->wordCount;
  double p = 0;
  for (j=0 ; j<J ; j++)
    p += info->asr_wordLevelFeatures[j][x23];
  if (J > 0)
    p /= J;
  return p;
}


/* Generic functions for context features.
 * Compute the context feature of the 'iFeat'-th feature in array 'features',
 * for the 'iWord'-th word. */
float xLeft(float **features, int iWord, int iFeat) {
  float **t = features;
  if (iWord == 0)
    return t[iWord][iFeat];
  else if (iWord == 1)
    return t[iWord-1][iFeat] * t[iWord][iFeat];
  else
    return t[iWord-2][iFeat] * t[iWord-1][iFeat] * t[iWord][iFeat];
}
float xCenter(float **features, int wordCount, int iWord, int iFeat) {
  float **t = features;
  if (iWord == 0)
    return t[iWord][iFeat] * t[iWord+1][iFeat];
  else if (iWord == wordCount-1)
    return t[iWord-1][iFeat] * t[iWord][iFeat];
  else
    return t[iWord-1][iFeat] * t[iWord][iFeat] * t[iWord+1][iFeat];
}
float xRight(float **features, int wordCount, int iWord, int iFeat) {
  float **t = features;
  if (iWord == wordCount-1)
    return t[iWord][iFeat];
  else if (iWord == wordCount-2)
    return t[iWord][iFeat] * t[iWord+1][iFeat];
  else
    return t[iWord][iFeat] * t[iWord+1][iFeat] * t[iWord+2][iFeat];
}

/* Context features,
   must be carefully placed after the corresponding feature in the global array */
float mt_x21_left(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xLeft(info->mt_wordLevelFeatures, i, set->mt_wlf[MT_X21].idx);}
float mt_x21_center(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xCenter(info->mt_wordLevelFeatures, info->target->wordCount, i, set->mt_wlf[MT_X21].idx);}
float mt_x21_right(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xRight(info->mt_wordLevelFeatures, info->target->wordCount, i, set->mt_wlf[MT_X21].idx);}
float mt_x22_left(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xLeft(info->mt_wordLevelFeatures, i, set->mt_wlf[MT_X22].idx);}
float mt_x22_center(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xCenter(info->mt_wordLevelFeatures, info->target->wordCount, i, set->mt_wlf[MT_X22].idx);}
float mt_x22_right(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xRight(info->mt_wordLevelFeatures, info->target->wordCount, i, set->mt_wlf[MT_X22].idx);}
float mt_x23_left(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xLeft(info->mt_wordLevelFeatures, i, set->mt_wlf[MT_X23].idx);}
float mt_x23_center(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xCenter(info->mt_wordLevelFeatures, info->target->wordCount, i, set->mt_wlf[MT_X23].idx);}
float mt_x23_right(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xRight(info->mt_wordLevelFeatures, info->target->wordCount, i, set->mt_wlf[MT_X23].idx);}

float asr_x21_left(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xLeft(info->asr_wordLevelFeatures, i, set->asr_wlf[ASR_X21].idx);}
float asr_x21_center(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xCenter(info->asr_wordLevelFeatures, info->source->wordCount, i, set->asr_wlf[ASR_X21].idx);}
float asr_x21_right(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xRight(info->asr_wordLevelFeatures, info->source->wordCount, i, set->asr_wlf[ASR_X21].idx);}
float asr_x22_left(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xLeft(info->asr_wordLevelFeatures, i, set->asr_wlf[ASR_X22].idx);}
float asr_x22_center(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xCenter(info->asr_wordLevelFeatures, info->source->wordCount, i, set->asr_wlf[ASR_X22].idx);}
float asr_x22_right(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xRight(info->asr_wordLevelFeatures, info->source->wordCount, i, set->asr_wlf[ASR_X22].idx);}
float asr_x23_left(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xLeft(info->asr_wordLevelFeatures, i, set->asr_wlf[ASR_X23].idx);}
float asr_x23_center(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xCenter(info->asr_wordLevelFeatures, info->source->wordCount, i, set->asr_wlf[ASR_X23].idx);}
float asr_x23_right(cm_resources_t *res, feature_set_t *set, translation_pair_t *info, int i)
{return xRight(info->asr_wordLevelFeatures, info->source->wordCount, i, set->asr_wlf[ASR_X23].idx);}

