#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#
# Copyright (C) 2011, Sylvain Raybaud <sylvain.raybaud@loria.fr>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Sylvain Raybaud nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Sylvain Raybaud <sylvain.raybaud@loria.fr> ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Sylvain Raybaud BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#

from optparse import OptionParser
from mytools import open_plain_or_gz,warn,error,verbose_msg,openfiles,closefiles,getline,debug
from phonetizer import Phonetizer
import sys

def get_40_words(ctm):
    W = []
    w = ctm.readline()
    while len(W)<40 and w!="":
        W.append(w.split()[4])
        w = ctm.readline()
    return W

def get_longest_match(l1,l2):
    i = 0
    j = 0
    while j<len(l2) and l1[0]!=l2[j]:
        j += 1
    if j==len(l2):
        return 0,0
    J = j
    while i<len(l1) and j<len(l2) and l1[:i+1]==l2[J:j+1]:
        i += 1
        j += 1
    return i,J

def merge(ph,p_):
    
    p = p_.split()
    i = 1
    best_i = 0
    best_match = 0
    best_offset = 0
    
    while i<len(p) and i<len(ph):
        longest_match,offset = get_longest_match(ph[len(ph)-i:],p[:i])
        if longest_match>best_match:
            best_match = longest_match
            best_i = i
            best_offset = offset
        i += 1
    
    res = (ph[:-best_i] if best_i>0 else ph)+p[best_offset:]
    return res
    

def phonetize_ctm(dicfname,bdlexfname,ctmfname):
    P = Phonetizer(dicfname,bdlexfname)
    
    ctm = open_plain_or_gz(ctmfname)
    
    ph = []
    
    W = get_40_words(ctm)
    buff = []
    while len(W)>0:
        p = P.phonetize_random_alt(" ".join(buff+W))
        debug(" ".join(p))
        ph = merge(ph,p)
        buff = W[-10:]
        W = get_40_words(ctm)
    
    print(" ".join(ph))
    
    ctm.close()

if __name__=='__main__':
    parser = OptionParser(usage="Usage: %prog [options] dic bdlex ctm > txt")
    parser.add_option('-v','--verbose',default=False,action="store_true",dest="verbose",help="verbosity [%default]")
    opts,args = parser.parse_args()
    if len(args)!=3:
        parser.print_help()
        error("bad command line.",1)
    
    phonetize_ctm(args[0],args[1],args[2])
