#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#
# Copyright (C) 2009, Sylvain Raybaud <sylvain.raybaud@crans.org>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of Sylvain Raybaud nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY Sylvain Raybaud <sylvain.raybaud@crans.org> ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Sylvain Raybaud BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#


from optparse import OptionParser
from mytools import dictree_ins,dictree_get,myFile,add,error,debug
from levenshtein import levenshtein
from random import choice,Random,random
from copy import deepcopy
import sys

Random()

EPSILON = '*EPS*'

RULES =[("temps en temps",[1,0]),
        ("temps universel",[0]),
        ("deux heures",[1]),
        ("trois heures",[1]),
        ("cinq heures",[1]),
        ("six heures",[1]),
        ("huit heures",[1]),
        ("petit à petit",[1,0]),
        ("non agression",[1]),
        ("pied à terre",[1,0]),
        ("premier avril",[1]),
        ("premier août",[1]),
        ("premier octobre",[1]),
        ("nez à nez",[-1,0,-1]),
        ("corps à corps",[-1,0]),
        ("nous",[1]),
        ("vous",[1]),
        ("mes",[1]),
        ("les",[1]),
        ("tes",[1]),
        ("ils",[1]),
        ("elles",[1]),
        ("on",[1]),
        ("en",[1]),
        ("leurs",[1]),
        ("certains",[1]),
        ("certaines",[1]),
        ("quelques",[1]),
        ("plusieurs",[1]),
        ("nombreux",[1]),
        ("aucun",[1]),
        ("et",[-1]),
        ("bon à",[-1])
        ]

class Bdlex_item:
    def __init__(self,word,is_adj=False,is_verb=False,is_noun=False,is_pronoun=False,is_plur=False):
        self.is_adj = is_adj
        self.is_verb = is_verb
        self.is_noun = is_noun
        self.is_pronoun = is_pronoun
        self.is_plur = is_plur

    def merge(self,is_adj=False,is_verb=False,is_noun=False,is_pronoun=False,is_plur=False):
        self.is_adj = self.is_adj or is_adj
        self.is_verb = self.is_verb or is_verb
        self.is_noun = self.is_noun or is_noun
        self.is_pronoun = self.is_pronoun or is_pronoun
        self.is_plur = self.is_plur or is_plur

class Rules:
    def __init__(self,bdlexfname=None,list=RULES):
        self.rules = {}
        self.bdlex = None
        for l in list:
            dictree_ins(self.rules,l[0].strip().split(),l[1])

        if bdlexfname:
            self.bdlex = {}
            f = myFile(bdlexfname)
            l = f.readline()
            while l!="":
                tokens = l.strip(";\t\n ").split(";")
                w = tokens[0].split("=")[1].strip()
                is_adj = False
                is_verb = False
                is_noun = False
                is_pronoun = False
                is_plur = False
                for t in tokens[1:]:
                    k,v = t.strip().split("=")
                    if k=="pos":
                        is_adj = v in ["adjnom","adjectif"]
                        is_verb = v in ["verbe","participepasse"]
                        is_noun = v in ["nom","adjnom"]
                        is_pronoun = v=="pronom"
                    elif k=="number":
                        is_plur = v=="plur"
                if w in self.bdlex:
                    self.bdlex[w].merge(is_adj,is_verb,is_noun,is_pronoun,is_plur)
                else:
                    self.bdlex[w] = Bdlex_item(w,is_adj,is_verb,is_noun,is_pronoun,is_plur)
                l = f.readline()
            f.close()

    def map_bdlex(self,tokens):
        if tokens[0] not in self.bdlex:
            return [0]
        tokens[0] = self.bdlex[tokens[0]]
        if tokens[0] in self.bdlex and tokens[0].is_noun and (not tokens[0].is_plur):
            return [-1]
        if tokens[1] not in self.bdlex:
            return [0]
        tokens[1] = self.bdlex[tokens[1]]
        if tokens[0].is_adj and tokens[1].is_noun:
            return [1]
        elif (tokens[0].is_pronoun or tokens[0]=="en") and tokens[1].is_verb:
            return [1]
        elif (tokens[1].is_pronoun or tokens[1] in ["en","y"]) and tokens[0].is_verb:
            return [1]
        else:
            return [0]

    def map_liaisons(self,tokens):
        if len(tokens)==0:
            return []
        if self.bdlex and len(tokens)==2:
            res_rules = dictree_get(self.rules,tokens)
            if not res_rules:
                res_rules = dictree_get(self.rules,tokens[:-1])
            if not res_rules:
                res_rules = []
            res_bdlex = self.map_bdlex(tokens)
            res = [0]*max(len(res_rules),len(res_bdlex))
            for i,l in enumerate(res_rules):
                res[i] = res_rules[i]
            for i,l in enumerate(res_bdlex):
                res[i] += res_bdlex[i]
            return res

        res = dictree_get(self.rules,tokens)
        return res if res else self.map_liaisons(tokens[:-1])
        #if res or len(tokens)==2:
            #return [res]*(len(tokens)-1) if res else [0]*(len(tokens)-1)
        #else:
            #return self.map_liaisons(tokens[:-1])


class Phonemes_lexicon:
    def __init__(self,dicfilename):
        self.filename = dicfilename
        self.lexicon = {} # grapheme (string) -> liaison_fin (bool) -> [pronunciation (string list)] (list)
        self.liaisons_debut = {} # grapheme -> bool
        self.phonemes = set()
        f = myFile(dicfilename)
        l = f.readline()
        while l!="":
            #debug(l)
            t = l.strip().split(" ",2)
            grapheme = t[0]
            liaisons = t[1].strip("[]")
            pronunciation = t[2]
            self.phonemes = self.phonemes.union(pronunciation.split())
            liaison_debut = liaisons[0]=='_'
            liaison_fin = liaisons[-1]=='_'
            p_list = dictree_get(self.lexicon,[grapheme,liaison_fin])
            if p_list:
                p_list.append(pronunciation)
            else:
                dictree_ins(self.lexicon,[grapheme,liaison_fin],[pronunciation])
            self.liaisons_debut[grapheme] = liaison_debut
            l = f.readline()
        f.close()

    def has_liaison_debut(self,grapheme):
        return (grapheme in self.liaisons_debut) and self.liaisons_debut[grapheme]

    def has_liaison_fin(self,grapheme):
        return (grapheme in self.lexicon) and (True in self.lexicon[grapheme])

    def get(self,grapheme,liaison_fin):
        return dictree_get(self.lexicon,[grapheme,liaison_fin]) if self.has_grapheme(grapheme) else None  #return nothing if grapheme not found

    def has_grapheme(self,gphm):
        return gphm in self.lexicon
    
    def get_phonemes(self):
        return self.phonemes

class Phonetizer:
    def __init__(self,dicfilename,bdlex):
        self.lexicon = Phonemes_lexicon(dicfilename)
        self.rules = Rules(bdlex)
    
    def get_phonemes(self):
        return self.lexicon.get_phonemes()

    def phonetize_random_alt(self,string):
        p_alts = self.phonetize_with_alts(string)
        p = ""
        for t in p_alts:
            p += " "+choice(t)
        return p

    def phonetize_expand_alts(self,string):
        p_alts = self.phonetize_with_alts(string)
        p_all = [""]
        for alts in p_alts:
            p_new = []
            for p in p_all:
                for a in alts:
                    p_new.append(p+a+" ")
            p_all = deepcopy(p_new)
        return p_all

    def phonetize_for_tt(self,string):
        p_alts = self.phonetize_with_alts_and_lastword_liaison(string)
        p_all = [""]
        for alts in p_alts:
            p_new = []
            for p in p_all:
                for a in alts:
                    p_new.append(p+a+" ")
            p_all = deepcopy(p_new)
        return p_all

    # technically, this can generate incorrect alternatives (among correct ones)
    #def phonetize_compact_alts(self,string):
        #p_items = self.phonetize_with_alts(string)
        #compact_alts = []

        #for alts in p_items:
            #c_alts = list(map(lambda x: [x],alts[0].split()))
            #for a in alts[1:]:
                #new_c_alts = []
                #_,_,c_alts_,_ = levenshtein(c_alts,a.split(),lambda x,y: y in x)
                #for i,new_alt in enumerate(c_alts_):
                    #new_alt = list(new_alt)
                    #if new_alt[1]==None:
                        #new_alt[1] = EPSILON
                    #new_item = new_alt[0] if new_alt[0]!=None else [EPSILON]
                    #if not (new_alt[1] in new_item):
                        #new_item.append(new_alt[1])
                    #new_c_alts.append(new_item)
                #c_alts = new_c_alts
            #compact_alts += c_alts

        #return compact_alts

    def phonetize_with_alts(self,string):
        tokens = string.strip().split()
        phonemes = []
        liaisons = [0]*len(tokens)
        liaisons[-1] = -99
        for i,t in enumerate(tokens):
            liaison_debut = (i>0 and liaisons[i-1]>=0 and self.lexicon.has_liaison_debut(t))
            if i<(len(tokens)-1):
                liaisons_rules = self.rules.map_liaisons(tokens[i:])
                for j,l in enumerate(liaisons_rules):
                    liaisons[i+j] += l
            liaison_fin = (liaisons[i]>=0 and i<(len(tokens)-1) and  self.lexicon.has_liaison_fin(t) and  self.lexicon.has_liaison_debut(tokens[i+1]))

            if not liaison_fin:
                liaisons[i] = -1

            alts = self.lexicon.get(t,liaison_fin)
            if alts:  #may be None if grapheme not found in lexicon
                if liaisons[i]==0 and liaison_fin:
                    alts = alts+self.lexicon.get(t,False)
                #p = choice(self.lexicon.get(t,liaison_fin))
                #phonemes += " "+" ".join(p)

                phonemes.append(alts)

        return phonemes

    def phonetize_with_alts_and_lastword_liaison(self,string):
        tokens = string.strip().split()
        phonemes = []
        liaisons = [0]*len(tokens)
        for i,t in enumerate(tokens):
            liaison_debut = (i>0 and liaisons[i-1]>=0 and self.lexicon.has_liaison_debut(t))
            if i<(len(tokens)-1):
                liaisons_rules = self.rules.map_liaisons(tokens[i:])
                for j,l in enumerate(liaisons_rules):
                    liaisons[i+j] += l
            liaison_fin = liaisons[i]>=0 and self.lexicon.has_liaison_fin(t) and  (i==(len(tokens)-1) or self.lexicon.has_liaison_debut(tokens[i+1]))

            if not liaison_fin:
                liaisons[i] = -1

            alts = self.lexicon.get(t,liaison_fin)
            if alts:  #may be None if grapheme not found in lexicon
                if liaisons[i]==0 and liaison_fin:
                    alts = alts+self.lexicon.get(t,False)
    
                phonemes.append(alts)

        return phonemes

if __name__ == "__main__":
    parser = OptionParser(usage="Usage:\n%prog [options] dicfile < in > out")
    parser.add_option('-b','--bdlex',default=None,metavar="FILE",dest="bdlex",help="use bdlex for more advanced rules [%default]")
    parser.add_option('-v','--verbose',default=False,action="store_true",dest="verbose",help="verbosity [%default]")
    opts,args = parser.parse_args()
    if len(args)!=1:
        parser.print_help()
        error("bad command line.",1)

    phonetizer = Phonetizer(args[0],opts.bdlex)

    l = sys.stdin.readline()
    while l!="":
        sys.stdout.write(phonetizer.phonetize_random_alt(l).strip()+"\n")
        l = sys.stdin.readline()

